#!/bin/bash

INFILE="/etc/mdtab"
OUTFILE="/etc/raidtab"

if [ -f $OUTFILE ] ; then
  mv $OUTFILE $OUTFILE.old
fi

parse_config () {
  case $1 in
        linear) LEVEL="linear" ;;
        raid0) LEVEL="0" ;;
        raid1) LEVEL="1" ;;
        *) echo "can't convert personality $1" ; return ;;
  esac
  echo "      raid-level $LEVEL" >> $OUTFILE
  echo "      persistent-superblock 0" >> $OUTFILE
  CHUNK=`echo $2 | sed -e "s/k$//"`
  echo "      chunk-size $CHUNK" >> $OUTFILE
  if [ $3 != "0" ] ; then
     echo "fault number $3 not used for $1 on $TARGET"
  fi
}

write_devlist () {
  NUM=0
  while [ $# -gt 0 ] ; do
    echo "      device $1" >> $OUTFILE
    echo "      raid-disk $NUM" >> $OUTFILE
    NUM=$(($NUM+1))
    shift
  done
}



parse_line () {
  TARGET=$1
  echo "raiddev $TARGET" >> $OUTFILE
  shift 
  CONFIG=`echo $1 | sed -e "s/,/ /g"`
  parse_config $CONFIG
  shift
  # rest should be devices (or comment)
  DEVICES=$*
  DEVLIST=""
  COMMENT=""
  NUMDEV=0
  while [ $# -gt 0 ] ; do
    case $1 in
      \/dev\/*) DEVLIST="$DEVLIST $1" ; NUMDEV=$(($NUMDEV+1)) ;;
      *) COMMENT="$COMMENT $1" ;;
    esac
    shift
  done
  echo "      nr-raid-disks $NUMDEV" >> $OUTFILE
  write_devlist $DEVLIST
  echo "# $COMMENT" >> $OUTFILE
  echo >> $OUTFILE
}


# here we go

echo "#" > $OUTFILE
echo "# raidtab automatically converted from old /etc/mdtab" >> $OUTFILE
echo "# better check for correctness" >> $OUTFILE
echo "#" >> $OUTFILE


while read line ; do
   case $line in
     \/dev\/*)
	parse_line $line
	;;
     *)
        echo "# $line" >> $OUTFILE
	;;
   esac
done  < $INFILE



